# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class DeviceComplianceUpdateModule(BaseUpdateModule):
    """A class used to update Intune Device Compliance

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceCompliancePolicies/"

    def __init__(self, *args, **kwargs):
        """Initializes the DeviceComplianceUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Policies/"
        self.assignment_endpoint = "/deviceManagement/deviceCompliancePolicies/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
            "root['scheduledActionsForRule']",
            "root['deviceComplianceScriptName']",
            "root['scheduledActionConfigurations']",
            "root['scheduledActionConfigurations'][0]['id']",
            "root['assignments']",
            "root['scheduledActionsForRule'][0]['scheduledActionConfigurations']",
        ]
        self.params = {
            "expand": "scheduledActionsForRule($expand=scheduledActionConfigurations)"
        }

    def _set_compliance_script_id(self, data: dict) -> dict[str, any]:
        compliance_script_id = self.make_graph_request(
            endpoint=self.endpoint + "/beta/deviceManagement/deviceComplianceScripts",
            params={
                "$filter": f"displayName eq '{data['deviceComplianceScriptName']}'"
            },
        )
        if compliance_script_id.get("value"):
            data["deviceCompliancePolicyScript"][
                "deviceComplianceScriptId"
            ] = compliance_script_id["value"][0]["id"]

            return data

        return False

    def _compliance_script_check(self, repo_data: dict) -> dict[str, any]:
        """Checks if the detection script exists

        Args:
            repo_data (dict): The data to check the detection script for
        """
        if "deviceComplianceScriptName" in repo_data:
            script_name = repo_data["deviceComplianceScriptName"]
            repo_data = self._set_compliance_script_id(repo_data)
            if repo_data is False:
                print(
                    f"Detection script {script_name} not found, Compliance Policy {self.name} not updated"
                )

        return repo_data

    def _scheduledActionsForRule_diff_check(self, repo_data: dict) -> None:
        """Checks the scheduled actions for the rule for differences

        Args:
            repo_data (dict): The data to check the scheduled actions for the rule for
        """
        for intune_action, repo_action in zip(
            self.downstream_object.get("scheduledActionsForRule"),
            repo_data["scheduledActionsForRule"],
        ):
            action_diff = self.get_diffs(repo_action, intune_action, None)

            if action_diff:
                data = {
                    "deviceComplianceScheduledActionForRules": [
                        {
                            "ruleName": "PasswordRequired",
                            "scheduledActionConfigurations": repo_data[
                                "scheduledActionsForRule"
                            ][0]["scheduledActionConfigurations"],
                        }
                    ]
                }
                self.update_downstream_data(
                    config_endpoint=self.endpoint
                    + self.CONFIG_ENDPOINT
                    + self.downstream_id
                    + "/scheduleActionsForRules",
                    method="post",
                    data=data,
                    status_code=200,
                )

                self.update_diff_data(action_diff)

    def _get_notification_template_id(self, rule: dict[str, any]) -> dict[str, any]:
        """Gets the notification template for a rule

        Args:
            rule (dict[str, any]): The rule to get the notification template for

        Returns:
            dict[str, any]: The notification template
        """
        for action in rule["scheduledActionConfigurations"]:
            if action.get("notificationTemplateName"):
                notification_template = self.make_graph_request(
                    self.endpoint
                    + "/beta/deviceManagement/notificationMessageTemplates/",
                    params={
                        "$filter": f"displayName eq '{action['notificationTemplateName']}'"
                    },
                )
                if notification_template["value"]:
                    action["notificationTemplateId"] = notification_template["value"][
                        0
                    ]["id"]
                else:
                    action[
                        "notificationTemplateId"
                    ] = "00000000-0000-0000-0000-000000000000"

                action.pop("notificationTemplateName")

        return rule

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                self.config_type = "Compliance Policy"
                self.notify = True
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    if repo_data.get("platforms") == "linux":
                        continue
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "@odata.type": repo_data.get("@odata.type"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    repo_data = self._compliance_script_check(repo_data)
                    if repo_data is False:
                        continue

                    for rule in repo_data.get("scheduledActionsForRule"):
                        self._get_notification_template_id(rule)

                    for item in intune_data["value"]:
                        for action in item["scheduledActionsForRule"][0][
                            "scheduledActionConfigurations"
                        ]:
                            self.remove_keys(action)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        self.notify = False
                        self.config_type = "Compliance Policy Scheduled Actions"
                        self._scheduledActionsForRule_diff_check(repo_data)

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
import json
import os
import re

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class EnrollmentConfigurationsUpdateModule(BaseUpdateModule):
    """A class used to update Intune Enrollment Configurations

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceEnrollmentConfigurations/"

    def __init__(self, *args, **kwargs):
        """Initializes the EnrollmentConfigurationsUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Configurations/"
        self.assignment_endpoint = "/deviceManagement/deviceEnrollmentConfigurations/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
            "root['priority']",
            "root['deviceEnrollmentConfigurationType']",
            "root['platformType']",
        ]
        self.assignment_key = "enrollmentConfigurationAssignments"

    def _get_config_type(self, repo_data: dict[str, any]) -> str:
        """Gets the configuration type

        Args:
            repo_data (dict[str, any]): The repository data

        Returns:
            str: The configuration type
        """
        config_type = repo_data.get("deviceEnrollmentConfigurationType", None)
        config_type = config_type[0].upper() + config_type[1:]
        config_type = re.findall("[A-Z][^A-Z]*", config_type)
        config_type = " ".join(config_type)

        return config_type

    def _update_priority(
        self,
        intune_object: dict[str, any],
        repo_data_priority: str,
        config_type: str,
    ) -> None:
        """Updates the priority

        Args:
            downstream_data (list[dict[str, any]]): The Intune data
            repo_data (dict[str, any]): The repository data
            config_type (str): The configuration type
        """
        if repo_data_priority == 0:
            return None
        # get priority from intune again to check if it was updated
        intune_object = self.make_graph_request(
            self.endpoint + self.CONFIG_ENDPOINT + self.downstream_id,
        )
        intune_priority = intune_object["priority"]
        if repo_data_priority != intune_priority:
            self.log(
                msg=f"Updating Enrollment Config {config_type} Priority: " + self.name,
            )
            # Update priority
            request_data = json.dumps({"priority": repo_data_priority})
            self.make_graph_request(
                self.endpoint
                + self.CONFIG_ENDPOINT
                + self.downstream_id
                + "/setpriority",
                data=request_data,
                status_code=200,
                method="post",
            )

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            intune_data["value"] = [
                item
                for item in intune_data["value"]
                if item["@odata.type"]
                != "#microsoft.graph.windows10EnrollmentCompletionPageConfiguration"
            ]

            for filename in os.listdir(self.path):
                self.downstream_id = None
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    if (
                        repo_data["@odata.type"]
                        == "#microsoft.graph.deviceEnrollmentPlatformRestrictionConfiguration"
                    ):
                        self.match_info = {
                            "displayName": repo_data.get("displayName"),
                            "@odata.type": repo_data.get("@odata.type"),
                            "platformType": repo_data.get("platformType"),
                        }
                    else:
                        self.match_info = {
                            "displayName": repo_data.get("displayName"),
                            "@odata.type": repo_data.get("@odata.type"),
                        }

                    self.name = repo_data.get("displayName")
                    config_type = self._get_config_type(repo_data)
                    self.config_type = f"Enrollment Configuration {config_type}"
                    repo_priority = repo_data.get("priority", None)
                    repo_data.pop("priority", None)
                    repo_data.pop("deviceEnrollmentConfigurationType", None)
                    repo_data.pop("platformType", None)
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        self._update_priority(
                            self.downstream_object, repo_priority, config_type
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            for item in list(intune_data["value"]):
                # Remove items that cannot or should be removed
                if (
                    item.get("@odata.type")
                    == "#microsoft.graph.windows10EnrollmentCompletionPageConfiguration"
                    or item.get("displayName") == "All users and all devices"
                ):
                    intune_data["value"].remove(item)

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class AutopilotDevicesBackupModule(BaseBackupModule):
    """A class used to backup Intune Autopilot Devices

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/windowsAutopilotDeviceIdentities"

    def __init__(self, *args, **kwargs):
        """Initializes the AutopilotDevicesBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Autopilot Devices/"
        self.append_id = False
        self.audit = False
        self.prefix = None
        # Autopilot Devices has no assignments, so exclude assignments from the run
        self.has_assignments = False

    def main(self) -> None:
        """The main method to backup the Autopilot Devices"""
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Autopilot Device data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        self.log(msg="Backing up Autopilot Devices")

        try:
            self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="id",
                log_message=None,
            )
        except Exception as e:
            self.log(tag="error", msg=f"Error processing Autopilot Device data: {e}")
            return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
          ..
        ....
       .::::
      .:::::            ___       _                     ____ ____
     .::::::           |_ _|_ __ | |_ _   _ _ __   ___ / ___|  _ \
    .:::::::.           | || '_ \| __| | | | '_ \ / _ \ |   | | | |
   ::::::::::::::.      | || | | | |_| |_| | | | |  __/ |___| |_| |
  ::::::::::::::.      |___|_| |_|\__|\__,_|_| |_|\___|\____|____/                 _
        :::::::.       |_ _|_ __ | |_ _   _ _ __   ___    __ _ ___    ___ ___   __| | ___
        ::::::.         | || '_ \| __| | | | '_ \ / _ \  / _` / __|  / __/ _ \ / _` |/ _ \
        :::::.          | || | | | |_| |_| | | | |  __/ | (_| \__ \ | (_| (_) | (_| |  __/
        ::::           |___|_| |_|\__|\__,_|_| |_|\___|  \__,_|___/  \___\___/ \__,_|\___|
        :::
        ::

This module contains the functions to run the backup.
"""

import argparse
import base64
import json
import os
import sys
from io import StringIO

from .backup_entra import backup_entra
from .backup_intune import backup_intune
from .intunecdlib.archive import move_to_archive
from .intunecdlib.get_accesstoken import obtain_azure_token
from .intunecdlib.get_authparams import getAuth

REPO_DIR = os.environ.get("REPO_DIR")


def start():
    parser = argparse.ArgumentParser(description="Save backup of Intune configurations")
    parser.add_argument(
        "-o",
        "--output",
        help="The format backups will be saved as, valid options are json or yaml. Default is json",
        type=str,
        default="json",
    )
    parser.add_argument(
        "-p",
        "--path",
        help="The path to which the configurations will be saved. Default value is $(Build.SourcesDirectory)",
        type=str,
        default=REPO_DIR,
    )
    parser.add_argument(
        "-m",
        "--mode",
        help=(
            "The mode in which the script is run, 0 = devtoprod (backup from dev -> update to prod) "
            "uses os.environ DEV_TENANT_NAME, DEV_CLIENT_ID, DEV_CLIENT_SECRET, "
            "1 = standalone (backup from prod) uses os.environ TENANT_NAME, CLIENT_ID, CLIENT_SECRET"
        ),
        type=int,
        default=0,
    )
    parser.add_argument(
        "-a",
        "--localauth",
        help=(
            "When this paramater is set, provide a path to a local dict file containing the following keys: "
            "params:TENANT_NAME, CLIENT_ID, CLIENT_SECRET when run in standalone mode and "
            "params:DEV_TENANT_NAME, DEV_CLIENT_ID, DEV_CLIENT_SECRET when run in devtoprod"
        ),
        type=str,
    )
    parser.add_argument(
        "-c",
        "--certauth",
        help="When using certificate auth, the following ENV variables is required: TENANT_NAME, CLIENT_ID, THUMBPRINT, KEY_FILE",
        action="store_true",
    )
    parser.add_argument(
        "-i",
        "--interactiveauth",
        help="When using interactive auth, the following ENV variables is required: TENANT_NAME, CLIENT_ID",
        action="store_true",
    )
    parser.add_argument(
        "-e",
        "--exclude",
        help="List of objects to exclude from the backup, separated by space.",
        choices=[
            "assignments",
            "AppConfigurations",
            "AppProtection",
            "APNs",
            "VPP",
            "Applications",
            "Compliance",
            "NotificationTemplate",
            "Profiles",
            "GPOConfigurations",
            "AppleEnrollmentProfile",
            "WindowsEnrollmentProfile",
            "EnrollmentStatusPage",
            "Filters",
            "ManagedGooglePlay",
            "Intents",
            "CompliancePartner",
            "ManagementPartner",
            "RemoteAssistancePartner",
            "ProactiveRemediation",
            "PowershellScripts",
            "ShellScripts",
            "ConfigurationPolicies",
            "ConditionalAccess",
            "EnrollmentConfigurations",
            "DeviceManagementSettings",
            "CustomAttributes",
            "DeviceCategories",
            "windowsDriverUpdates",
            "windowsFeatuteUpdates",
            "windowsQualityUpdates",
            "Roles",
            "ScopeTags",
            "VPPusedLicenseCount",
            "GPlaySyncTime",
            "CompliancePartnerHeartbeat",
            "DeviceCompliancePolicies",
            "ComplianceScripts",
            "ReusablePolicySettings",
            "entraApplications",
            "entraAuthenticationFlowsPolicy",
            "entraAuthenticationMethods",
            "entraAuthorizationPolicy",
            "entraB2BPolicy",
            "entraDeviceRegistrationPolicy",
            "entraExternalIdentitiesPolicy",
            "entraGroupSettings",
            "entraRoamingSettings",
            "entraSecurityDefaults",
            "entraSSPR",
            "entraUserSettings",
            "entraDomains",
        ],
        nargs="+",
    )
    parser.add_argument(
        "--intunecdmonitor",
        help="When this parameter is set, the script is run in the IntuneCDMonitor context",
        action="store_true",
    )
    parser.add_argument(
        "-ap",
        "--autopilot",
        help="If set, a record of autopilot devices will be saved",
        action="store_true",
    )
    parser.add_argument(
        "--prefix",
        help="When set, only backs up configurations whose name starts with the configured prefix",
        type=str,
    )
    parser.add_argument(
        "--append-id",
        help="When set, the id of the configuration will be appended to the name of the exported file",
        action="store_true",
    )
    parser.add_argument(
        "--entrabackup",
        help="When set, backs up Entra configurations",
        action="store_true",
    )
    parser.add_argument(
        "--ignore-omasettings",
        help="When set, ignores encrypted OMA Settings configuration type. Useful if you only want read permissions to Graph API.",
        action="store_true",
    )
    parser.add_argument(
        "--activationlock",
        help="When set, backs up Activation Lock Bypass Codes",
        action="store_true",
    )
    parser.add_argument(
        "--scopes",
        help="The scopes to use when obtaining an access token interactively separated by space. Only used when using interactive auth. Default is: DeviceManagementApps.ReadWrite.All, DeviceManagementConfiguration.ReadWrite.All, DeviceManagementManagedDevices.Read.All, DeviceManagementServiceConfig.ReadWrite.All, DeviceManagementRBAC.ReadWrite.All, Group.Read.All, Policy.ReadWrite.ConditionalAccess, Policy.Read.All",
        nargs="+",
    )
    parser.add_argument(
        "-v", "--verbose", help="Prints verbose output", action="store_true"
    )
    parser.add_argument(
        "--audit",
        help="When set, the script will process the audit data from Intune and commit the changes to the git repo using the name of the user who made the change and the date and time of the change",
        action="store_true",
    )
    parser.add_argument(
        "-t",
        "--token",
        help="The authentication token to use for the update if not using an app registration",
        type=str,
    )
    parser.add_argument(
        "--exit-on-error",
        help="When set, the script will exit on the first error",
        action="store_true",
    )

    args = parser.parse_args()

    if args.verbose:
        os.environ["VERBOSE"] = "True"

    if args.exit_on_error:
        os.environ["EXIT_ON_ERROR"] = "True"

    def devtoprod():
        return "devtoprod"

    def standalone():
        return "standalone"

    switcher = {0: devtoprod, 1: standalone}

    def selected_mode(argument):
        func = switcher.get(argument, "nothing")
        return func()

    if args.certauth or args.interactiveauth:
        args.mode = None
    else:
        args.mode = selected_mode(args.mode)

    if not args.scopes:
        args.scopes = [
            "DeviceManagementApps.ReadWrite.All",
            "DeviceManagementConfiguration.ReadWrite.All",
            "DeviceManagementManagedDevices.Read.All",
            "DeviceManagementServiceConfig.ReadWrite.All",
            "DeviceManagementRBAC.ReadWrite.All",
            "Group.Read.All",
            "Policy.ReadWrite.ConditionalAccess",
            "Policy.Read.All",
        ]

    if not args.token:
        token = getAuth(
            args.mode,
            args.localauth,
            args.certauth,
            args.interactiveauth,
            args.scopes,
            args.entrabackup,
            tenant="DEV",
        )
    else:
        token = {"access_token": args.token}

    if args.entrabackup:
        azure_token = obtain_azure_token(os.environ.get("TENANT_ID"), args.path)

    def run_backup(path, output, exclude, token, prefix, append_id):
        results = []

        if args.entrabackup:
            print("***Entra backup***")

            backup_entra(results, path, output, token, azure_token, args, exclude)

            print("***Intune backup***")

        backup_intune(results, path, output, exclude, token, prefix, append_id, args)

        from .intunecdlib.assignment_report import AssignmentReport

        AssignmentReport(path, output).main()

        results = [result for result in results if result is not None]

        config_count = sum([result.get("config_count", 0) for result in results])

        created_files = [
            output
            for result in results
            if result.get("outputs", None)
            for output in result.get("outputs", None)
            if output is not None
        ]

        move_to_archive(path, created_files, output)

        return config_count

    if args.output == "json" or args.output == "yaml":
        if token is None:
            raise Exception("Token is empty, please check os.environ variables")

        if args.exclude:
            exclude = args.exclude
        else:
            exclude = []

        if args.intunecdmonitor:
            old_stdout = sys.stdout
            sys.stdout = feedstdout = StringIO()
            count = run_backup(
                args.path, args.output, exclude, token, args.prefix, args.append_id
            )
            sys.stdout = old_stdout
            feed_bytes = feedstdout.getvalue().encode("utf-8")
            out = base64.b64encode(feed_bytes).decode("utf-8")

            summary = {"config_count": count, "feed": out}

            with open(f"{args.path}/backup_summary.json", "w") as f:
                f.write(json.dumps(summary))

        else:
            run_backup(
                args.path, args.output, exclude, token, args.prefix, args.append_id
            )

    else:
        print("Please enter a valid output format, json or yaml")

    if "VERBOSE" in os.environ:
        del os.environ["VERBOSE"]

    if "EXIT_ON_ERROR" in os.environ:
        del os.environ["EXIT_ON_ERROR"]


if __name__ == "__main__":
    start()



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class AppProtectionUpdateModule(BaseUpdateModule):
    """A class used to update Intune App Protections

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
    """

    CONFIG_ENDPOINT = "/beta/deviceAppManagement/"

    def __init__(self, *args, **kwargs):
        """Initializes the AppProtectionUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/App Protection/"
        self.config_type = "App Protection"
        self.assignment_endpoint = "/deviceAppManagement/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['displayName']",
            "root['assignments']",
        ]
        self.app_protection = True
        self.assignment_status_code = 204
        self.remove_status_code = 204

    def _get_platform(self, repo_data: dict) -> str:
        """Gets the platform of the App Protection

        Args:
            repo_data (dict): The data of the App Protection

        Returns:
            str: The platform of the App Protection
        """
        if (
            repo_data["@odata.type"]
            == "#microsoft.graph.mdmWindowsInformationProtectionPolicy"
        ):
            platform = "mdmWindowsInformationProtectionPolicies"
        elif (
            repo_data["@odata.type"]
            == "#microsoft.graph.windowsInformationProtectionPolicy"
        ):
            platform = "windowsInformationProtectionPolicies"
        else:
            platform = f"{str(repo_data['@odata.type']).split('.')[2]}s"

        return platform

    def _get_match_info(self, repo_data: dict) -> dict[str, any]:
        """Gets the match info of the App Protection

        Args:
            repo_data (dict): The data of the App Protection

        Returns:
            dict[str, any]: The match info of the App Protection
        """
        if (
            "targetedAppManagementLevels" in repo_data
            and repo_data["targetedAppManagementLevels"] != "unspecified"
        ):
            match_info = {
                "displayName": repo_data.get("displayName"),
                "targetedAppManagementLevels": repo_data.get(
                    "targetedAppManagementLevels"
                ),
            }
        else:
            match_info = {
                "displayName": repo_data.get("displayName"),
                "@odata.type": repo_data.get("@odata.type"),
            }

        return match_info

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(
                    self.CONFIG_ENDPOINT + "managedAppPolicies"
                )
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                self.assignment_endpoint = "/deviceAppManagement/"
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = self._get_match_info(repo_data)
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    if (
                        repo_data["@odata.type"]
                        == "#microsoft.graph.windowsInformationProtectionPolicy"
                    ):
                        status_code = 200
                    else:
                        status_code = 204

                    # Get the platform of the App Protection
                    platform = self._get_platform(repo_data)

                    self.assignment_endpoint = f"{self.assignment_endpoint}{platform}/"
                    config_endpoint = f"{self.CONFIG_ENDPOINT}{platform}/"

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=status_code,
                            config_endpoint=config_endpoint,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(
                self.CONFIG_ENDPOINT + "managedAppPolicies/", intune_data["value"]
            )

        return self.diff_summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module moves files to archive during backup if they have been removed from Intune.
"""

import os
import shutil
from datetime import datetime

# Folders to exclude from archving
exclude = set(
    [
        "Management Intents",
        "archive",
        "__archive__",
        "Assignment Report",
        "Autopilot Devices",
    ]
)
# Date tag for archive folder
date_tag = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")


def archive(path, file, root) -> None:
    """Moves a file to the archive folder.

    Args:
        path (_str_): path to current folder
        file (_str_): file to archive
        root (_str_): root of the file
    """
    if not os.path.exists(f"{path}/__archive__/{date_tag}"):
        os.makedirs(f"{path}/__archive__/{date_tag}")
    shutil.move(
        os.path.join(root, file),
        os.path.join(path, f"__archive__/{date_tag}", file),
    )


def move_to_archive(path, created_files, output) -> None:
    """Moves a file to the archive folder.

    Args:
        path (_str_): path to current folder
        created_files (_list_): list of created files during backup
        output (_str_): format the file is in
    """
    if not os.path.exists(f"{path}/__archive__"):
        os.makedirs(f"{path}/__archive__/")

    for root, dirs, files in os.walk(path, topdown=True):
        # Remove excluded folders from dirs
        dirs[:] = [d for d in dirs if d not in exclude]
        for file in files:
            # if json is in root, skip it and move on
            if file.endswith(".json") and root == path:
                continue
            if file.endswith(".yaml") or file.endswith(".json"):
                # if file is not in created_files, archive it
                if file.replace(f".{output}", "") not in created_files:
                    archive(path, file, root)

    # Check if Management Intents folder exists
    if os.path.exists(f"{path}/Management Intents") is True:
        for root, dirs, files in os.walk(f"{path}/Management Intents", topdown=True):
            for file in files:
                if file.endswith(".yaml") or file.endswith(".json"):
                    # if file is not in created_files, archive it
                    if file.replace(f".{output}", "") not in created_files:
                        archive(path, file, root)



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class B2BPolicyBackupModule(BaseBackupModule):
    """A class used to backup Entra B2B Policy

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "B2B/b2bPolicy"

    def __init__(self, *args, **kwargs):
        """Initializes the B2BPolicyBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Entra/External Collaboration Settings/"
        self.preset_filename = "b2b_policy"

    def main(self) -> dict[str, any]:
        """The main method to backup the Entra data

        Returns:
            dict[str, any]: The results of the backup
        """

        try:
            self.entra_data = self.make_azure_request(self.CONFIG_ENDPOINT)
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting External Collaboration Settings data from {self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        self.log(msg="Backing up Entra External Collaboration Settings B2B Policy")

        try:
            self.results = self.process_data(
                data=self.entra_data,
                filetype=self.filetype,
                path=self.path,
                name_key="",
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error processing External Collaboration Settings data: {e}",
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ProactiveRemediationUpdateModule(BaseUpdateModule):
    """A class used to update Intune Proactive Remediation

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceHealthScripts/"

    def __init__(self, *args, **kwargs):
        """Initializes the ProactiveRemediationUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Proactive Remediations/"
        self.script_data_path = f"{self.path}Script Data/"
        self.assignment_endpoint = "/deviceManagement/deviceHealthScripts/"
        self.assignment_extra_url = "/assign"
        self.assignment_key = "deviceHealthScriptAssignments"

    def _get_script_data(self, filename: str) -> tuple[str, str]:
        fname_id = filename.split("__")
        detection_script_name = ""
        remediation_script_name = ""

        if len(fname_id) > 1:
            fname_id = fname_id[1].replace(".json", "").replace(".yaml", "")
        else:
            fname_id = ""

        # Get all remediation scripts and detection scripts files
        script_files = os.listdir(self.script_data_path)

        # Filter out files that matches the id
        script_files = [f for f in script_files if fname_id in f]

        # Set detection and remediation script name and path
        for f in script_files:
            if "detectionscript" in f.lower():
                detection_script_name = f"{self.script_data_path}{f}"
            elif "remediationscript" in f.lower():
                remediation_script_name = f"{self.script_data_path}{f}"

        if detection_script_name and remediation_script_name:
            detection_script = self.load_script_file(detection_script_name)
            remediation_script = self.load_script_file(remediation_script_name)
            return detection_script, remediation_script

        return "", ""

    def _handle_script_diff(
        self, repo_script: str, intune_script: str, repo_data: dict[str, any]
    ) -> None:
        """Handle the script diff

        Args:
            script (str): The script to handle
        """
        diff = self.get_diffs(repo_script, intune_script)

        if diff:
            self.update_downstream_data(
                config_endpoint=self.endpoint
                + self.CONFIG_ENDPOINT
                + self.downstream_id,
                method="patch",
                data=repo_data,
                status_code=200,
            )

            self.update_diff_data(diff)

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            profile_ids = [profile["id"] for profile in intune_data["value"]]

            remediation_data = self.batch_request(
                profile_ids,
                "deviceManagement/deviceHealthScripts/",
                "",
            )

            for filename in os.listdir(self.path):
                self.config_type = "Proactive Remediation"
                self.notify = True
                self.exclude_paths = [
                    "root['assignments']",
                    "root['detectionScriptContent']",
                    "root['remediationScriptContent']",
                    "root['deviceHealthScriptType']",
                ]

                repo_data = self.load_repo_data(filename)
                if repo_data:
                    repo_data.pop("deviceHealthScriptType", None)
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    (
                        repo_detection_script,
                        repo_remediation_script,
                    ) = self._get_script_data(filename)

                    repo_data["detectionScriptContent"] = self.encode_base64(
                        repo_detection_script
                    )
                    repo_data["remediationScriptContent"] = self.encode_base64(
                        repo_remediation_script
                    )

                    try:
                        self.process_update(
                            downstream_data=remediation_data,
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        self.notify = False
                        intune_detection_script = self.decode_base64(
                            self.downstream_object.get("detectionScriptContent")
                        )
                        intune_remediation_script = self.decode_base64(
                            self.downstream_object.get("remediationScriptContent")
                        )
                        self.config_type = "Detection Script"
                        self.message = (
                            "Detection Script changed, check commit history for details"
                        )
                        self.exclude_paths = [
                            "root['assignments']",
                            "root['remediationScriptContent']",
                        ]
                        self._handle_script_diff(
                            repo_detection_script, intune_detection_script, repo_data
                        )

                        self.config_type = "Remediation Script"
                        self.message = "Remediation Script changed, check commit history for details"
                        self.exclude_paths = [
                            "root['assignments']",
                            "root['detectionScriptContent']",
                        ]
                        self._handle_script_diff(
                            repo_remediation_script,
                            intune_remediation_script,
                            repo_data,
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            for item in list(remediation_data):
                if item.get("publisher") == "Microsoft":
                    remediation_data.remove(item)
            self.remove_downstream_data(self.CONFIG_ENDPOINT, remediation_data)

        return self.diff_summary



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class EnrollmentStatusPageUpdateModule(BaseUpdateModule):
    """A class used to update Intune Enrollment Status Page

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceEnrollmentConfigurations/"
    APP_ENDPOINT = "/beta/deviceAppManagement/mobileApps"

    def __init__(self, *args, **kwargs):
        """Initializes the EnrollmentStatusPageUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Profiles/Windows/ESP/"
        self.config_type = "Enrollment Status Page"
        self.assignment_endpoint = "/deviceManagement/deviceEnrollmentConfigurations/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
            "root['priority']",
        ]
        self.assignment_key = "enrollmentConfigurationAssignments"

    def _get_app_ids(self, repo_data: dict) -> dict[str, any]:
        """Get the app IDs for the selected apps

        Args:
            repo_data (dict): The data from the repo

        Returns:
            dict[str, any]: The updated data
        """
        if "selectedMobileAppNames" in repo_data:
            app_ids = []

            for app in repo_data["selectedMobileAppNames"]:
                param = {
                    "$filter": f"(isof('{str(app['type']).replace('#', '')}'))",
                    "$search": '"' + app["name"] + '"',
                }

                app_request = self.make_graph_request(
                    endpoint=self.endpoint + self.APP_ENDPOINT, params=param
                )
                if app_request["value"]:
                    app_ids.append(app_request["value"][0]["id"])
                else:
                    self.log(tag="warning", msg=f"App {app['name']} not found")

            if app_ids:
                repo_data.pop("selectedMobileAppNames", None)
                repo_data["selectedMobileAppIds"] = app_ids

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "@odata.type": repo_data.get("@odata.type"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    self._get_app_ids(repo_data)
                    repo_data.pop("priority", None)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            for item in list(intune_data["value"]):  # Create a copy for iteration
                # Remove items that cannot or should be removed
                if (
                    item.get("@odata.type")
                    != "#microsoft.graph.windows10EnrollmentCompletionPageConfiguration"
                    or item.get("displayName") == "All users and all devices"
                ):
                    intune_data["value"].remove(item)

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
def update_intune(
    diff_summary,
    path,
    token,
    assignment,
    report,
    create_groups,
    remove,
    exclude,
    args,
):
    """
    Imports all the update functions and runs them
    """
    params = {
        "path": path,
        "token": token,
        "exclude": exclude,
        "report": report,
        "remove": remove,
        "create_groups": create_groups,
        "handle_assignment": assignment,
    }

    from .intunecdlib.process_scope_tags import ProcessScopeTags

    process_scope_tags = ProcessScopeTags(token)

    if "ScopeTags" not in exclude:
        from .update.Intune.ScopeTags import ScopeTagsUpdateModule

        diff_summary.append(ScopeTagsUpdateModule(**params).main())
        scope_tags = process_scope_tags.get_scope_tags()
    else:
        scope_tags = None

    params["scope_tags"] = scope_tags

    if "Filters" not in exclude:
        from .update.Intune.Filters import FiltersUpdateModule

        diff_summary.append(FiltersUpdateModule(**params).main())

    if "AppConfigurations" not in exclude:
        from .update.Intune.AppConfiguration import AppConfigurationUpdateModule

        diff_summary.append(AppConfigurationUpdateModule(**params).main())

    if "AppProtection" not in exclude:
        from .update.Intune.AppProtection import AppProtectionUpdateModule

        diff_summary.append(AppProtectionUpdateModule(**params).main())

    if "NotificationTemplate" not in exclude:
        from .update.Intune.NotificationTemplate import NotificationTemplateUpdateModule

        diff_summary.append(NotificationTemplateUpdateModule(**params).main())

    if "ReusablePolicySettings" not in exclude:
        from .update.Intune.ReusableSettings import ReusableSettingsUpdateModule

        diff_summary.append(ReusableSettingsUpdateModule(**params).main())

    if "ComplianceScripts" not in exclude:
        from .update.Intune.ComplianceScripts import ComplianceScriptsUpdateModule

        diff_summary.append(ComplianceScriptsUpdateModule(**params).main())

    if "DeviceCompliancePolicies" not in exclude:
        from .update.Intune.Compliance import ComplianceUpdateModule

        diff_summary.append(ComplianceUpdateModule(**params).main())

    if "Compliance" not in exclude:
        from .update.Intune.DeviceCompliance import DeviceComplianceUpdateModule

        diff_summary.append(DeviceComplianceUpdateModule(**params).main())

    if "DeviceManagementSettings" not in exclude and args.interactiveauth is True:
        from .update.Intune.DeviceManagementSettings import (
            DeviceManagementSettingsUpdateModule,
        )

        diff_summary.append(DeviceManagementSettingsUpdateModule(**params).main())
    else:
        print(
            "***Device Management Settings is only available with interactive auth***"
        )

    if "DeviceCategories" not in exclude:
        from .update.Intune.DeviceCategories import DeviceCategoriesUpdateModule

        diff_summary.append(DeviceCategoriesUpdateModule(**params).main())

    if "Profiles" not in exclude:
        from .update.Intune.DeviceConfigurations import DeviceConfigurationsUpdateModule

        diff_summary.append(DeviceConfigurationsUpdateModule(**params).main())

    if "GPOConfigurations" not in exclude:
        from .update.Intune.GroupPolicyConfigurations import (
            GroupPolicyConfigurationsUpdateModule,
        )

        diff_summary.append(GroupPolicyConfigurationsUpdateModule(**params).main())

    if "AppleEnrollmentProfile" not in exclude:
        from .update.Intune.AppleEnrollmentProfile import (
            AppleEnrollmentProfileUpdateModule,
        )

        diff_summary.append(AppleEnrollmentProfileUpdateModule(**params).main())

    if "WindowsEnrollmentProfile" not in exclude:
        from .update.Intune.WindowsEnrollmentProfile import (
            WindowsEnrollmentProfileUpdateModule,
        )

        diff_summary.append(WindowsEnrollmentProfileUpdateModule(**params).main())

    if "EnrollmentStatusPage" not in exclude:
        from .update.Intune.EnrollmentStatusPage import EnrollmentStatusPageUpdateModule

        diff_summary.append(EnrollmentStatusPageUpdateModule(**params).main())

    if "EnrollmentConfigurations" not in exclude:
        from .update.Intune.EnrollmentConfigurations import (
            EnrollmentConfigurationsUpdateModule,
        )

        diff_summary.append(EnrollmentConfigurationsUpdateModule(**params).main())

    if "Intents" not in exclude:
        from .update.Intune.ManagementIntents import ManagementIntentsUpdateModule

        diff_summary.append(ManagementIntentsUpdateModule(**params).main())

    if "ProactiveRemediation" not in exclude:
        from .update.Intune.ProactiveRemediation import ProactiveRemediationUpdateModule

        diff_summary.append(ProactiveRemediationUpdateModule(**params).main())

    if "PowershellScripts" not in exclude:
        from .update.Intune.PowerShellScripts import PowerShellScriptsUpdateModule

        diff_summary.append(PowerShellScriptsUpdateModule(**params).main())

    if "ShellScripts" not in exclude:
        from .update.Intune.ShellScripts import ShellScriptsUpdateModule

        diff_summary.append(ShellScriptsUpdateModule(**params).main())

    if "CustomAttribute" not in exclude:
        from .update.Intune.CustomAttributes import CustomAttributesUpdateModule

        diff_summary.append(CustomAttributesUpdateModule(**params).main())

    if "ConfigurationPolicies" not in exclude:
        from .update.Intune.SettingsCatalog import SettingsCatalogUpdateModule

        diff_summary.append(SettingsCatalogUpdateModule(**params).main())

    if "ConditionalAccess" not in exclude:
        from .update.Intune.ConditionalAccess import ConditionalAccessUpdateModule

        diff_summary.append(ConditionalAccessUpdateModule(**params).main())

    if "WindowsDriverUpdateProfiles" not in exclude:
        from .update.Intune.WindowsDriverUpdates import WindowsDriverUpdatesUpdateModule

        diff_summary.append(WindowsDriverUpdatesUpdateModule(**params).main())

    if "windowsFeatureUpdates" not in exclude:
        from .update.Intune.WindowsFeatureUpdates import (
            WindowsFeatureUpdatesUpdateModule,
        )

        diff_summary.append(WindowsFeatureUpdatesUpdateModule(**params).main())

    if "windowsQualityUpdates" not in exclude:
        from .update.Intune.WindowsQualityUpdates import (
            WindowsQualityUpdatesUpdateModule,
        )

        diff_summary.append(WindowsQualityUpdatesUpdateModule(**params).main())

    if "Roles" not in exclude:
        from .update.Intune.Roles import RolesUpdateModule

        diff_summary.append(RolesUpdateModule(**params).main())



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class AppleEnrollmentProfileUpdateModule(BaseUpdateModule):
    """A class used to update Intune Apple Enrollment Profiles

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/depOnboardingSettings/"

    def __init__(self, *args, **kwargs):
        """Initializes the AppleEnrollmentProfileUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Profiles/Apple/"
        self.config_type = "Apple Enrollment Profile"
        self.exclude_paths = ["root['isDefault']"]
        self.handle_assignment = False
        self.create_config = False

    def _get_profile_item_and_account_id(
        self, profile_data: dict, repo_data: dict
    ) -> tuple[dict, str]:
        """Gets the profile item and account ID of the Apple Enrollment Profile

        Args:
            profile_data (dict): The data of the Apple Enrollment Profile
            repo_data (dict): The data of the Apple Enrollment Profile in the repository

        Returns:
            tuple[dict, str]: The profile item and account ID of the Apple Enrollment Profile
        """
        profile_item = None
        account_id = None
        for profile in profile_data:
            if profile["displayName"] == repo_data["displayName"]:
                profile_item = profile
                account_id = profile_item["id"].split("_")[0]
                break

        if not account_id or not profile_item:
            return None, None

        return profile_item, account_id

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            profile_ids = [profile["id"] for profile in intune_data["value"]]

            batch_data = self.batch_request(
                profile_ids,
                "deviceManagement/depOnboardingSettings/",
                "/enrollmentProfiles",
            )
            profile_data = [
                value
                for profile in batch_data
                for value in profile["value"]
                if value is not None
            ]

            for filename in os.listdir(self.path):
                self.downstream_id = None
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    repo_data.pop("isDefault", None)
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    profile_item, account_id = self._get_profile_item_and_account_id(
                        profile_data, repo_data
                    )

                    if not profile_item or not account_id:
                        self.log(
                            tag="error",
                            msg=f"Error getting {self.config_type} {self.name} item or account ID",
                        )
                        continue

                    self.downstream_id = profile_item["id"]
                    config_endpoint = (
                        f"{self.CONFIG_ENDPOINT}{account_id}/enrollmentProfiles/"
                    )

                    try:
                        self.process_update(
                            downstream_data=profile_data,
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=config_endpoint,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

        return self.diff_summary
